package com.github.jfcloud.gene.sample.service.biz;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.github.jfcloud.admin.api.sys.dto.message.MessageUserDTO;
import com.github.jfcloud.gene.common.util.CustomIdGenerator;
import com.github.jfcloud.gene.common.util.UserUtil;
import com.github.jfcloud.gene.constants.GeneStatusEnum;
import com.github.jfcloud.gene.flow.service.NotifyService;
import com.github.jfcloud.gene.sample.entity.SampleFood;
import com.github.jfcloud.gene.sample.entity.SampleFoodDetail;
import com.github.jfcloud.gene.sample.entity.SampleInfo;
import com.github.jfcloud.gene.sample.mapper.SampleFoodMapper;
import com.github.jfcloud.gene.sample.mapper.SampleInfoMapper;
import com.github.jfcloud.gene.sample.service.SampleEditService;
import com.github.jfcloud.gene.sample.service.SampleFoodDetailService;
import com.github.jfcloud.gene.sample.service.SampleFoodService;
import com.github.jfcloud.gene.sample.vo.SampleFoodDetailVo;
import com.github.jfcloud.gene.sample.vo.SampleFoodVo;
import com.github.jfcloud.gene.sample.vo.SampleSubmitVo;
import com.github.jfcloud.gene.sys.service.DBSystemPropertiesService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class SampleFoodServiceImpl extends ServiceImpl<SampleFoodMapper, SampleFood> implements SampleFoodService, SampleEditService {

    private final SampleInfoMapper sampleInfoMapper;
    private final SampleFoodDetailService sampleFoodDetailService;
    private final CommonSampleEditServiceImpl commonSampleEditService;
    private final DBSystemPropertiesService systemPropertiesService;
    private final NotifyService notifyService;

    @Override
    public void removeBySampleId(Long sampleId) {
        List<SampleFood> sampleFoods = list(new LambdaQueryWrapper<>(SampleFood.class)
                .select(SampleFood::getId)
                .eq(SampleFood::getSampleId, sampleId));
        if (sampleFoods.isEmpty()) {
            return;
        }

        Set<Long> foodIdSet = sampleFoods.stream().map(SampleFood::getId).collect(Collectors.toSet());
        removeBatchByIds(foodIdSet);

        //删除详情
        sampleFoodDetailService.remove(new LambdaQueryWrapper<>(SampleFoodDetail.class)
                .in(SampleFoodDetail::getSampleFoodId, foodIdSet));
    }

    @Override
    public void save(Long sampleId, SampleSubmitVo vo) {
        Assert.notNull(vo.getAnimalFood(), "宠物食品送检不能为空");

        //先删除之前的记录
        removeBySampleId(sampleId);

        //保存动物食品送检信息
        SampleFood sampleFood = BeanUtil.copyProperties(vo.getAnimalFood(), SampleFood.class);
        sampleFood.setId(CustomIdGenerator.nextId());
        sampleFood.setSampleId(sampleId);
        sampleFood.insert();

        //保存详情
        List<SampleFoodDetailVo> detailList = vo.getAnimalFood().getDetailList();
        if (!detailList.isEmpty()) {
            List<SampleFoodDetail> sampleFoodDetails = BeanUtil.copyToList(detailList, SampleFoodDetail.class);
            sampleFoodDetails.forEach(detail -> detail.setSampleFoodId(sampleFood.getId()));
            sampleFoodDetailService.saveBatch(sampleFoodDetails);
        }

        generate(sampleId);
    }

    @Override
    public void generate(Long sampleId) {

    }

    @Override
    public void afterAudit(Long sampleId) {
        SampleInfo sampleInfo = sampleInfoMapper.selectById(sampleId);
        if (sampleInfo == null) {
            return;
        }

        SampleEditService.super.afterAudit(sampleId);

        //如果审核完成，提醒送检人和收样人
        if (GeneStatusEnum.COMPLETED.getStatus().equals(sampleInfo.getStatus())) {
            String now = DateUtil.format(new Date(), DatePattern.CHINESE_DATE_TIME_PATTERN);
            String dingMsg = String.format("【%s】【%s/%s】%s同意，请执行项目", sampleInfo.projectType(),
                    GeneStatusEnum.PROJECT_MANAGEMENT.getDescription(), UserUtil.getNickName(), now);

            //查询是否需要钉钉通知
            String key = String.format("sample.%s.dingding.enable", sampleInfo.getStatus());
            if (systemPropertiesService.getBooleanValue(key)) {
                SampleFoodVo detail = getDetail(sampleId);

                List<MessageUserDTO> messageUsers = new ArrayList<>();
                MessageUserDTO user1 = new MessageUserDTO();
                user1.setUserId(detail.getSenderContactId());
                messageUsers.add(user1);
                MessageUserDTO user2 = new MessageUserDTO();
                user2.setUserId(detail.getReceiverId());
                messageUsers.add(user2);
                notifyService.sendDingding("样本送检", dingMsg, messageUsers);
            }
            return;
        }

        commonSampleEditService.afterAudit(sampleId);
    }

    @Override
    public SampleFoodVo getDetail(Long id) {
        SampleFood sampleFood = getById(id);
        if (sampleFood == null) {
            return null;
        }
        List<SampleFoodDetail> detailList = sampleFoodDetailService.list(new LambdaQueryWrapper<>(SampleFoodDetail.class).eq(SampleFoodDetail::getSampleFoodId, id));

        SampleFoodVo sampleFoodVo = BeanUtil.copyProperties(sampleFood, SampleFoodVo.class);
        sampleFoodVo.setDetailList(BeanUtil.copyToList(detailList, SampleFoodDetailVo.class));
        return sampleFoodVo;
    }
}
